@@ -3,7 +3,7 @@ from mpi4py import MPI as _MPI
 
 from jax import abstract_arrays, core
 from jax.core import Primitive
-from jax.interpreters import ad, xla
+from jax.interpreters import ad, xla, batching
 from jax.lax import create_token
 from jax.lib import xla_client
 
@@ -32,7 +32,7 @@ mpi_allreduce_impl = default_primitive_impl(mpi_allreduce_p)
     comm=(type(None), _MPI.Intracomm, HashableMPIType),
     token=(type(None), xla.Token, core.Tracer),
 )
-def allreduce(x, op, comm=None, token=None):
+def allreduce(x, op, *, comm=None, token=None):
     """Perform an allreduce operation.
 
     .. note::
@@ -62,7 +62,7 @@ def allreduce(x, op, comm=None, token=None):
 
     op = wrap_as_hashable(op)
     comm = wrap_as_hashable(comm)
-    return mpi_allreduce_p.bind(x, token, op=op, comm=comm, transpose=False)
+    return tuple(mpi_allreduce_p.bind(x, token, op=op, comm=comm, transpose=False))
 
 
 # This function compiles the operation
@@ -154,37 +154,51 @@ def mpi_allreduce_abstract_eval(xs, token, op, comm, transpose):
     )
 
 
+def mpi_allreduce_batch_eval(in_args, batch_axes, op, comm, transpose):
+    x, token = in_args
+    res = mpi_allreduce_p.bind(x, token, op=op, comm=comm, transpose=transpose)
+    return res, batch_axes
+
+
 def mpi_allreduce_value_and_jvp(in_args, tan_args, op, comm, transpose):
     x, token = in_args
     x_tan, token_tan = tan_args
 
-    res = mpi_allreduce_p.bind(x, token, op=op, comm=comm, transpose=transpose)
-
     if unpack_hashable(op) != _MPI.SUM:
         raise NotImplementedError(
             "The adjoint of allreduce is only defined for op=MPI.SUM"
         )
 
-    jvp = mpi_allreduce_p.bind(x_tan, token, op=op, comm=comm, transpose=transpose)
-    return (res, jvp)
+    val, token = mpi_allreduce_p.bind(x, token, op=op, comm=comm, transpose=transpose)
+
+    # throw away return token to work around jax#6285
+    jvp, token_jvp = mpi_allreduce_p.bind(
+        x_tan, token, op=op, comm=comm, transpose=transpose
+    )
+    return (val, token), (jvp, ad.Zero.from_value(token_jvp))
 
 
 def mpi_allreduce_transpose_rule(tan_args, *x_args, op, comm, transpose):
     _, token = x_args
-    t, _ = tan_args
+    x_tan, token_tan = tan_args
 
     if unpack_hashable(op) != _MPI.SUM:
         raise NotImplementedError(
             "The linear transpose of allreduce is only defined for op=MPI.SUM"
         )
 
-    return mpi_allreduce_p.bind(t, token, op=op, comm=comm, transpose=(not transpose))
+    res, token = mpi_allreduce_p.bind(
+        x_tan, token, op=op, comm=comm, transpose=(not transpose)
+    )
+    return res, token_tan
 
 
 mpi_allreduce_p.multiple_results = True
 mpi_allreduce_p.def_impl(mpi_allreduce_impl)
 mpi_allreduce_p.def_abstract_eval(mpi_allreduce_abstract_eval)
 
+batching.primitive_batchers[mpi_allreduce_p] = mpi_allreduce_batch_eval
+
 ad.primitive_jvps[mpi_allreduce_p] = mpi_allreduce_value_and_jvp
 ad.primitive_transposes[mpi_allreduce_p] = mpi_allreduce_transpose_rule
 
