@@ -33,7 +33,7 @@ mpi_reduce_impl = default_primitive_impl(mpi_reduce_p)
     comm=(type(None), _MPI.Intracomm, HashableMPIType),
     token=(type(None), xla.Token, core.Tracer),
 )
-def reduce(x, op, root, comm=None, token=None):
+def reduce(x, op, root, *, comm=None, token=None):
     """Perform a reduce operation.
 
     Arguments:
@@ -65,9 +65,9 @@ def reduce(x, op, root, comm=None, token=None):
     res, token = mpi_reduce_p.bind(x, token, op=op, root=root, comm=comm)
 
     if rank != root:
-        return x, token
+        return (x, token)
 
-    return res, token
+    return (res, token)
 
 
 # This function compiles the operation
