@@ -3,7 +3,7 @@ from mpi4py import MPI as _MPI
 
 from jax import abstract_arrays, core
 from jax.core import Primitive
-from jax.interpreters import xla
+from jax.interpreters import ad, xla, batching
 from jax.lax import create_token
 from jax.lib import xla_client
 
@@ -42,6 +42,7 @@ def sendrecv(
     recvbuf,
     source,
     dest,
+    *,
     sendtag=0,
     recvtag=_MPI.ANY_TAG,
     comm=None,
@@ -85,26 +86,51 @@ def sendrecv(
     if status is not None:
         status = wrap_as_hashable(status)
 
-    return mpi_sendrecv_p.bind(
-        sendbuf,
-        recvbuf,
-        token,
-        source=source,
-        dest=dest,
-        sendtag=sendtag,
-        recvtag=recvtag,
-        comm=comm,
-        status=status,
+    return tuple(
+        mpi_sendrecv_p.bind(
+            sendbuf,
+            recvbuf,
+            token,
+            source=source,
+            dest=dest,
+            sendtag=sendtag,
+            recvtag=recvtag,
+            comm=comm,
+            status=status,
+            _must_transpose=False,
+        )
     )
 
 
 # This function compiles the operation
 @translation_rule_cpu
 def mpi_sendrecv_xla_encode_cpu(
-    c, sendbuf, recvbuf, token, source, dest, sendtag, recvtag, comm, status
+    c,
+    sendbuf,
+    recvbuf,
+    token,
+    source,
+    dest,
+    sendtag,
+    recvtag,
+    comm,
+    status,
+    _must_transpose=False,
 ):
     from ..xla_bridge.mpi_xla_bridge import MPI_STATUS_IGNORE_ADDR
 
+    # when performing forward diff, the gradient will follow the sent message.
+    # so if you do a sendrecv from rank 0 to 1, the gradient wrt the inputs of rank 0
+    # will end up in rank 1.\
+    # it's maybe possible to fix this by, at the end of the calculation, bringing back
+    # the gradient to the correct rank, but that would require some study.
+    if _must_transpose:
+        raise RuntimeError(
+            "sendrecv cannot be used with forward-mode (vjp) autodiff, because "
+            "the gradient might be located on a different mpi rank than the "
+            "desired one. Use reverse-mode (jvp) differentiation instead."
+        )
+
     comm = unpack_hashable(comm)
     status = unpack_hashable(status)
 
@@ -162,8 +188,26 @@ def mpi_sendrecv_xla_encode_cpu(
 
 @translation_rule_gpu
 def mpi_sendrecv_xla_encode_gpu(
-    c, sendbuf, recvbuf, token, source, dest, sendtag, recvtag, comm, status
+    c,
+    sendbuf,
+    recvbuf,
+    token,
+    source,
+    dest,
+    sendtag,
+    recvtag,
+    comm,
+    status,
+    _must_transpose=False,
 ):
+
+    if _must_transpose:
+        raise RuntimeError(
+            "sendrecv cannot be used with forward-mode (vjp) autodiff, because "
+            "the gradient might be located on a different mpi rank than the "
+            "desired one. Use reverse-mode (jvp) differentiation instead."
+        )
+
     from ..xla_bridge.mpi_xla_bridge import MPI_STATUS_IGNORE_ADDR
     from ..xla_bridge.mpi_xla_bridge_gpu import build_sendrecv_descriptor
 
@@ -226,7 +270,16 @@ def mpi_sendrecv_xla_encode_gpu(
 
 # This function evaluates only the shapes during AST construction
 def mpi_sendrecv_abstract_eval(
-    sendbuf, recvbuf, token, source, dest, sendtag, recvtag, comm, status
+    sendbuf,
+    recvbuf,
+    token,
+    source,
+    dest,
+    sendtag,
+    recvtag,
+    comm,
+    status,
+    _must_transpose=False,
 ):
     return (
         abstract_arrays.ShapedArray(recvbuf.shape, recvbuf.dtype),
@@ -234,10 +287,112 @@ def mpi_sendrecv_abstract_eval(
     )
 
 
+def mpi_sendrecv_batch_eval(
+    in_args,
+    batch_axes,
+    source,
+    dest,
+    sendtag,
+    recvtag,
+    comm,
+    status,
+    _must_transpose=False,
+):
+
+    sendbuf, recvbuf, token = in_args
+
+    assert batch_axes[0] == batch_axes[1]
+
+    res = mpi_sendrecv_p.bind(
+        sendbuf,
+        recvbuf,
+        token,
+        source=source,
+        dest=dest,
+        sendtag=sendtag,
+        recvtag=recvtag,
+        comm=comm,
+        status=status,
+        _must_transpose=_must_transpose,
+    )
+    return res, (batch_axes[0], batch_axes[2])
+
+
+def mpi_sendrecv_value_and_jvp(
+    in_args,
+    tan_args,
+    source,
+    dest,
+    sendtag,
+    recvtag,
+    comm,
+    status,
+    _must_transpose=False,
+):
+    sendbuf, recvbuf, token = in_args
+    send_tan, recv_tan, token_tan = tan_args
+
+    val, token = mpi_sendrecv_p.bind(
+        sendbuf,
+        recvbuf,
+        token,
+        source=source,
+        dest=dest,
+        sendtag=sendtag,
+        recvtag=recvtag,
+        comm=comm,
+        status=status,
+        _must_transpose=_must_transpose,
+    )
+
+    # throw away return token to work around jax#6285
+    jvp, token_jvp = mpi_sendrecv_p.bind(
+        send_tan,
+        recv_tan,
+        token,
+        source=source,
+        dest=dest,
+        sendtag=sendtag,
+        recvtag=recvtag,
+        comm=comm,
+        status=status,
+        _must_transpose=not _must_transpose,
+    )
+
+    return (val, token), (jvp, ad.Zero.from_value(token_jvp))
+
+
+def mpi_sendrecv_transpose_rule(
+    tan_args, *x_args, source, dest, sendtag, recvtag, comm, status, _must_transpose
+):
+    _, _, token = x_args
+    out_tan, token_tan = tan_args
+
+    # swap the sender and receiver
+    res, token = mpi_sendrecv_p.bind(
+        out_tan,
+        out_tan,
+        token,
+        source=dest,
+        dest=source,
+        sendtag=sendtag,
+        recvtag=recvtag,
+        comm=comm,
+        status=status,
+        _must_transpose=not _must_transpose,
+    )
+    return res, ad.Zero.from_value(res), token_tan
+
+
 mpi_sendrecv_p.multiple_results = True
 mpi_sendrecv_p.def_impl(mpi_sendrecv_impl)
 mpi_sendrecv_p.def_abstract_eval(mpi_sendrecv_abstract_eval)
 
+batching.primitive_batchers[mpi_sendrecv_p] = mpi_sendrecv_batch_eval
+
+ad.primitive_jvps[mpi_sendrecv_p] = mpi_sendrecv_value_and_jvp
+ad.primitive_transposes[mpi_sendrecv_p] = mpi_sendrecv_transpose_rule
+
 # assign to the primitive the correct encoder
 xla.backend_specific_translations["cpu"][mpi_sendrecv_p] = mpi_sendrecv_xla_encode_cpu
 xla.backend_specific_translations["gpu"][mpi_sendrecv_p] = mpi_sendrecv_xla_encode_gpu
